//------------------------------------------------------------------------------
// JsonFse - Copyright (c) 2015 - 2016 by Friedemann Seebass, Germany.
// Diese Bibliothek ist freie Software. Siehe GNU LGPL Notiz in der Datei
// JsonParser.java.
// This library is free software. See the GNU LGPL notice in file
// JsonParser.java.
//------------------------------------------------------------------------------

package JsonFse;

//------------------------------------------------------------------------------

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 Represents a JSON object. It holds the root node of a JSON tree, allows
 traversing through the pairs in the tree and accessing individual pairs.
 @author Friedemann Seebass
 */// fse, 18.10.2015

public class JsonObj
{
	/** The root node of the JSON tree. */
	protected NodeObj mNodeObj;
		
	//---------------------------------------------------------------------------
	/**
	Creates an empty JSON object to be filled later by adding pairs.
	*/// fse, 28.12.2016
	
	public JsonObj()
	{
		JsonParser jp = new JsonParser();

		try
		{
			mNodeObj = jp.createTree("{}");
		}
		catch (ParseErrorEx ex)
		{
			Logger.getLogger(JsonObj.class.getName()).log(Level.SEVERE, null, ex);
		}
	}
	
	//---------------------------------------------------------------------------
	/**
	Parses the given string into a recursive JSON tree.
	@param str the JSON object to parse as String
	@throws ParseErrorEx Is thrown if the parser detects an error in the string.
	*/// fse, 18.10.2015
	
	public JsonObj(String str) throws ParseErrorEx
	{
		JsonParser jp = new JsonParser();

		mNodeObj = jp.createTree(str);
	}
	
	//---------------------------------------------------------------------------
	/**
	Returns the root node of the JSON tree.
	@return The root node.
	*/// fse, 18.10.2015
	
	public NodeObj getNodeObj() { return mNodeObj; }

	//---------------------------------------------------------------------------
	/**
	Returns the JSON tree as formatted string.
	@return The whole JSON object as formatted string.
	*/// fse, 18.10.2015
	
	public String getAsString() { return mNodeObj.getAsString(0, false); }
	
	//---------------------------------------------------------------------------
	/**
	Gets the next pair in the JSON object.
	@return The next pair in the JSON object.
	*/// fse, 18.10.2015
	
	public NodePair getNext() { return mNodeObj.getNext(); }
	
	//---------------------------------------------------------------------------
	/**
	Gets the pair with index idx of the JSON object.
	@param idx index of the pair to retrieve
	@return The pair with index idx or null if index out of range.
	*/// fse, 18.10.2015
	
	public NodePair get(int idx) { return mNodeObj.get(idx); }
	
	//---------------------------------------------------------------------------
	/**
	Gets the value of the pair with the string (i. e. the label) given in the
	parameter and converts it to a string. The search starts at index 0, and only
	the first pair is found if	there should be more than one with the same label.
	@param string The label to search.
	@return The value of the label as a string or null if not found.
	*/// fse, 06.11.2015

	public String getValueAsString(String string)
	{
		NodeValue value = getValue(string);
	
		return value == null ? null : value.getAsString();
	}
	
	//---------------------------------------------------------------------------
	/**
	Gets the value of the pair with the string (i. e. the label) given in the
	parameter. The search starts at index 0, and only the first pair is found if
	there should be more than one with the same label.
	@param string The label to search.
	@return The value of the label or null if not found.
	*/// fse, 06.11.2015

	public NodeValue getValue(String string)
	{
		int       idx   = mNodeObj.mNodeMembers.mIdx;
		NodeValue value = null;
		NodePair  np;
		
		if (string.charAt(0) != '\"')
			string = "\"" + string + "\"";
		mNodeObj.mNodeMembers.mIdx = 0;
		np = getNext();
		while (np != null)
		{
			if (np.mString.getAsString().equals(string))
			{
				value = np.mValue;
				break;
			}
			np = getNext();
		}
		mNodeObj.mNodeMembers.mIdx = idx;
		return value;
	}
	
	//---------------------------------------------------------------------------
	/**
	Adds a new pair where the value is a string to the end of the node's
	member list.
	@param string The pair's label.
	@param value  The pair's value.
	*/// fse, 21.12.2016
	
	public void addPair(String string, String value)
	{
		NodePair np = new NodePair(string, value);
		mNodeObj.add(np);
	}
	
	//---------------------------------------------------------------------------
	/**
	Adds a new pair where the value is a number to the end of the node's
	member list.
	@param string The pair's label.
	@param value  The pair's value.
	*/// fse, 22.12.2016
	
	public void addPair(String string, double value)
	{
		NodePair np = new NodePair(string, value);
		mNodeObj.add(np);
	}
	
	//---------------------------------------------------------------------------
	/**
	Adds a new pair where the value is a boolean to the end of the node's
	member list.
	@param string The pair's label.
	@param value  The pair's value.
	*/// fse, 22.12.2016
	
	public void addPair(String string, boolean value)
	{
		NodePair np = new NodePair(string, value);
		mNodeObj.add(np);
	}
	
	//---------------------------------------------------------------------------
	/**
	Adds a new pair where the value is null to the end of the node's
	member list.
	@param string The pair's label.
	*/// fse, 22.12.2016
	
	public void addPair(String string)
	{
		NodePair np = new NodePair(string);
		mNodeObj.add(np);
	}
	
	//---------------------------------------------------------------------------
	/**
	Adds a new pair where the value is a NodeArray to the end of the node's
	member list.
	@param string The pair's label.
	@param value  The pair's value.
	*/// fse, 22.12.2016
	
	public void addPair(String string, NodeArray value)
	{
		NodePair np = new NodePair(string, value);
		mNodeObj.add(np);
	}
	
	//---------------------------------------------------------------------------
	/**
	Adds a new pair where the value is another JsonObj to the end of the node's
	member list.
	@param string The pair's label.
	@param value  The pair's value.
	*/// fse, 22.12.2016
	
	public void addPair(String string, JsonObj value)
	{
		NodePair np = new NodePair(string, value);
		mNodeObj.add(np);
	}
	
	//---------------------------------------------------------------------------

	public static void main(String args[])
	{
		String s = "{" +
							"\"name\" : \"Mike\"," +
							"\"age\"  : 23,"       +
                       "\"hobbies\" : [ \"fishing\", \"rugby\", \"computer games\" ]" +
					  "}";
		 try
		 {
			 JsonObj jo = new JsonObj(s);
			 System.out.println(jo.getAsString());
		 }
		 catch (ParseErrorEx ex)
		 {
			 System.out.println(ex.getErrMsg());
		 }
		 // Create an empty JSON object:
		 JsonObj jo = new JsonObj();
		 // Create the NodeArray:
		 NodeArray na = new NodeArray();
		 na.add(new NodeString("fishing"));
		 na.add(new NodeString("rugby"));
		 na.add(new NodeString("computer games"));
		 // Add the pairs:
		 jo.addPair("name", "Mike");
		 jo.addPair("age", 23);
		 jo.addPair("hobbies", na);
		 System.out.println(jo.getAsString());
	}
}

//---------------------------------------------------------------------------
//------------------------------------- -------------------------------------
//------------------------ ------------------------- ------------------------
//------------------ ------------------ ------------------ ------------------
//-------------- -------------- --------------- -------------- --------------
